@@ -198,13 +198,11 @@ json_gobject_new (GType       gtype,
   members = json_object_get_members (object);
   members_left = NULL;
 
-  /* first pass: construct and construct-only properties; here
-   * we cannot use Serializable because we don't have an
-   * instance yet; we use the default implementation of
-   * json_deserialize_pspec() to deserialize known types
+  /* first pass: construct-only properties; here we cannot use Serializable
+   * because we don't have an instance yet; we use the default implementation
+   * of json_deserialize_pspec() to deserialize known types
    *
-   * FIXME - find a way to allow deserialization for these
-   * properties
+   * FIXME - find a way to allow deserialization for these properties
    */
   construct_params = g_array_sized_new (FALSE, FALSE, sizeof (GParameter), n_members);
 
@@ -232,7 +230,12 @@ json_gobject_new (GType       gtype,
       val = json_object_get_member (object, member_name);
       res = json_deserialize_pspec (&param.value, pspec, val);
       if (!res)
-        g_value_unset (&param.value);
+	{
+	  g_warning ("Failed to deserialize \"%s\" property of type \"%s\" for an object of type \"%s\"",
+		     pspec->name, G_VALUE_TYPE_NAME (&param.value), g_type_name (gtype));
+
+	  g_value_unset (&param.value);
+	}
       else
         {
           param.name = g_strdup (pspec->name);
@@ -302,8 +305,7 @@ json_gobject_new (GType       gtype,
         continue;
 
       /* we should have dealt with these above */
-      if ((pspec->flags & G_PARAM_CONSTRUCT_ONLY) ||
-          (pspec->flags & G_PARAM_CONSTRUCT))
+      if (pspec->flags & G_PARAM_CONSTRUCT_ONLY)
         continue;
 
       if (!(pspec->flags & G_PARAM_WRITABLE))
@@ -400,13 +402,6 @@ json_gobject_dump (GObject *gobject)
       else
         g_object_get_property (gobject, pspec->name, &value);
 
-      /* skip if the value is the default for the property */
-      if (g_param_value_defaults (pspec, &value))
-        {
-          g_value_unset (&value);
-          continue;
-        }
-
       /* if there is a serialization vfunc, then it is completely responsible
        * for serializing the property, possibly by calling the implementation
        * of the default JsonSerializable interface through chaining up
@@ -417,7 +412,8 @@ json_gobject_dump (GObject *gobject)
                                             &value,
                                             pspec);
         }
-      else
+      /* skip if the value is the default for the property */
+      else if (!g_param_value_defaults (pspec, &value))
         node = json_serialize_pspec (&value, pspec);
 
       if (node)
@@ -517,35 +513,77 @@ json_deserialize_pspec (GValue     *value,
         {
         case G_TYPE_BOOLEAN:
         case G_TYPE_INT64:
-        case G_TYPE_DOUBLE:
         case G_TYPE_STRING:
-          g_value_copy (&node_value, value);
-          retval = TRUE;
+	  if (G_VALUE_HOLDS (&node_value, G_VALUE_TYPE (value)))
+	    {
+	      g_value_copy (&node_value, value);
+	      retval = TRUE;
+	    }
           break;
 
         case G_TYPE_INT:
-          g_value_set_int (value, (gint) g_value_get_int64 (&node_value));
-          retval = TRUE;
+	  if (G_VALUE_HOLDS (&node_value, G_TYPE_INT64))
+	    {
+	      g_value_set_int (value, (gint) g_value_get_int64 (&node_value));
+	      retval = TRUE;
+	    }
           break;
 
         case G_TYPE_CHAR:
-          g_value_set_char (value, (gchar) g_value_get_int64 (&node_value));
-          retval = TRUE;
+	  if (G_VALUE_HOLDS (&node_value, G_TYPE_INT64))
+	    {
+#if GLIB_CHECK_VERSION (2, 31, 0)
+	      g_value_set_schar (value, (gchar) g_value_get_int64 (&node_value));
+#else
+              g_value_set_char (value, (gchar) g_value_get_int64 (&node_value));
+#endif
+	      retval = TRUE;
+	    }
           break;
 
         case G_TYPE_UINT:
-          g_value_set_uint (value, (guint) g_value_get_int64 (&node_value));
-          retval = TRUE;
+	  if (G_VALUE_HOLDS (&node_value, G_TYPE_INT64))
+	    {
+	      g_value_set_uint (value, (guint) g_value_get_int64 (&node_value));
+	      retval = TRUE;
+	    }
           break;
 
         case G_TYPE_UCHAR:
-          g_value_set_uchar (value, (guchar) g_value_get_int64 (&node_value));
-          retval = TRUE;
+	  if (G_VALUE_HOLDS (&node_value, G_TYPE_INT64))
+	    {
+	      g_value_set_uchar (value, (guchar) g_value_get_int64 (&node_value));
+	      retval = TRUE;
+	    }
+          break;
+
+        case G_TYPE_DOUBLE:
+
+	  if (G_VALUE_HOLDS (&node_value, G_TYPE_DOUBLE))
+	    {
+	      g_value_set_double (value, g_value_get_double (&node_value));
+	      retval = TRUE;
+	    }
+	  else if (G_VALUE_HOLDS (&node_value, G_TYPE_INT64))
+	    {
+	      g_value_set_double (value, (gdouble) g_value_get_int64 (&node_value));
+	      retval = TRUE;
+	    }
+
           break;
 
         case G_TYPE_FLOAT:
-          g_value_set_float (value, (gfloat) g_value_get_double (&node_value));
-          retval = TRUE;
+	  if (G_VALUE_HOLDS (&node_value, G_TYPE_DOUBLE))
+	    {
+	      g_value_set_float (value, (gfloat) g_value_get_double (&node_value));
+	      retval = TRUE;
+	    }
+	  else if (G_VALUE_HOLDS (&node_value, G_TYPE_INT64))
+	    {
+	      g_value_set_float (value, (gfloat) g_value_get_int64 (&node_value));
+	      retval = TRUE;
+	    }
+
           break;
 
         case G_TYPE_ENUM:
@@ -710,7 +748,11 @@ json_serialize_pspec (const GValue *real_value,
 
     case G_TYPE_CHAR:
       retval = json_node_new (JSON_NODE_VALUE);
+#if GLIB_CHECK_VERSION (2, 31, 0)
+      json_node_set_int (retval, g_value_get_schar (real_value));
+#else
       json_node_set_int (retval, g_value_get_char (real_value));
+#endif
       break;
 
     case G_TYPE_UCHAR:
